<?php

namespace Drp\NativeEventApp\Test\Unit\Modules;

use Drp\NativeEventApp\Facades\ModulesUnSerializer;
use Drp\NativeEventApp\Models\Segment;
use Drp\NativeEventApp\Modules\MissingModule;
use Drp\NativeEventApp\Modules\Modules;
use Drp\NativeEventApp\Test\Support\Fakes\Modules\ContainerModule;
use Drp\NativeEventApp\Test\Support\Fakes\Modules\FakeModule;
use Drp\NativeEventApp\Test\Support\Fakes\Modules\ModuleWithDependency;
use Drp\NativeEventApp\Test\Support\InteractsWithModules;
use Drp\NativeEventApp\Test\TestCase;

class ModulesUnSerializerTest extends TestCase
{
    use InteractsWithModules;

    /**s
     * @test
     */
    public function module_can_be_serialized()
    {
        $module = new FakeModule();
        $module->text('CONTENT');
        $this->assertEquals('{"type":"fake_module","content":{"text":"CONTENT","other":null}}', $module->toJson());
    }

    /**
     * @test
     */
    public function modules_can_be_serialized()
    {
        $modules = Modules::make([
            FakeModule::make('heading')->text('CONTENT'),
            FakeModule::make('heading 2')->text('OTHER'),
        ]);

        $this->assertEquals(
            '[{"type":"fake_module","text":"CONTENT","other":null,"styles":[],"className":[],"viewableBy":[]},' .
            '{"type":"fake_module","text":"OTHER","other":null,"styles":[],"className":[],"viewableBy":[]}]',
            $modules->toJson()
        );
    }

    /**
     * @test
     */
    public function modules_can_be_un_serialized()
    {
        $modules = ModulesUnSerializer::getModulesFromArray([
            [
                'type' => 'fake_module',
                'name' => 'heading',
                'text' => 'CONTENT',
            ]
        ]);

        tap($modules->first(), function (FakeModule $module) {
            $this->assertEquals('CONTENT', $module->text);
            $this->assertNull($module->getOther());
        });
    }

    /**
     * @test
     */
    public function module_can_be_un_serialized_with_dependencies()
    {
        $modules = ModulesUnSerializer::getModulesFromArray([
            [
                'type' => 'module_with_dependency',
                'name' => 'heading',
                'other' => 'CONTENT',
                'text' => 'CONTENT',
            ]
        ]);

        tap($modules->first(), function (ModuleWithDependency $module) {
            $this->assertEquals('CONTENT', $module->text);
        });
    }

    /**
     * @test
     */
    public function modules_can_be_un_serialized_from_json_string()
    {
        $modules = ModulesUnSerializer::getModulesFromString(
            '[{"type":"fake_module","name":"heading","text":"CONTENT"}]'
        );

        tap($modules->first(), function (FakeModule $module) {
            $this->assertEquals('CONTENT', $module->text);
        });
    }

    /**
     * @test
     */
    public function handles_invalid_module_name()
    {
        $modules = ModulesUnSerializer::getModulesFromArray([
            [
                'type' => 'OTHER',
                'name' => 'heading',
                'text' => 'CONTENT',
            ]
        ]);

        tap($modules->first(), function (MissingModule $module) {
            $this->assertEquals('CONTENT', $module->text);
        });
    }

    /**
     * @test
     */
    public function handles_missing_module_name()
    {
        $modules = ModulesUnSerializer::getModulesFromArray([
            [
                'text' => 'CONTENT',
            ]
        ]);

        tap($modules->first(), function (MissingModule $module) {
            $this->assertEquals('CONTENT', $module->text);
        });
    }

    /**
     * @test
     */
    public function container_modules_can_be_serialized()
    {
        $container = ContainerModule::make('parent')->children(
            Modules::make([
                FakeModule::make('child')->text('CONTENT')
            ])
        );

        $modules = Modules::make([
            $container,
            FakeModule::make('other')->text('CONTENT')
        ]);

        $serializedModules = $modules->toJson();

        $this->assertEquals(json_encode([
            [
                'type' => 'container_module',
                'children' => [
                    [
                        'type' => 'fake_module',
                        'text' => 'CONTENT',
                        'other' => null,
                        'styles' => [],
                        'className' => [],
                        'viewableBy' => [],
                    ],
                ],
                'styles' => [],
                'className' => [],
                'viewableBy' => [],
            ],
            [
                'type' => 'fake_module',
                'text' => 'CONTENT',
                'other' => null,
                'styles' => [],
                'className' => [],
                'viewableBy' => [],
            ],
        ]), $serializedModules);

        //Assert the serialized modules can be un serialized
        $this->assertEquals($modules, ModulesUnSerializer::getModulesFromString($serializedModules));
    }

    /** @test */
    public function handles_modules_being_empty_string()
    {
        $modules = ModulesUnSerializer::getModulesFromString('');

        $this->assertEquals([], $modules->toArray());
    }

    /** @test */
    public function segments_can_be_serialized()
    {
        $module = new FakeModule();
        $module->text('CONTENT')
            ->viewableBy(collect([factory(Segment::class)->make(['id' => 1])]));
        $this->assertEquals(
            '{"type":"fake_module","text":"CONTENT","other":null,"styles":[],"className":[],"viewableBy":[1]}',
            $module->toJson()
        );
    }
}
